package com.qmth.sop.server.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.sop.business.bean.dto.OrgDto;
import com.qmth.sop.business.bean.params.LoginParam;
import com.qmth.sop.business.bean.result.EditResult;
import com.qmth.sop.business.bean.result.LoginResult;
import com.qmth.sop.business.bean.result.MenuResult;
import com.qmth.sop.business.cache.CommonCacheService;
import com.qmth.sop.business.entity.BasicAttachment;
import com.qmth.sop.business.entity.SysConfig;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBTask;
import com.qmth.sop.business.service.*;
import com.qmth.sop.business.util.ImportExportUtil;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.*;
import com.qmth.sop.common.util.Result;
import com.qmth.sop.common.util.ResultUtil;
import com.qmth.sop.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 系统公用 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2023-07-17
 */
@Api(tags = "系统公用Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_COMMON)
public class SysController {
    private final static Logger log = LoggerFactory.getLogger(SysController.class);

    @Resource
    SysUserService sysUserService;

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    ImportExportUtil importExportUtil;

    @Resource
    TBTaskService tbTaskService;

    @Resource
    SysUserRoleService sysUserRoleService;

    @Resource
    BasicVerifyCodeService basicVerifyCodeService;

    @Resource
    SysOrgService sysOrgService;

    @ApiOperation(value = "登录")
    @RequestMapping(value = "/login", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "用户信息", response = LoginResult.class)})
    @Aac(auth = BOOL.FALSE)
    public Result login(@ApiParam(value = "用户信息", required = true) @Valid @RequestBody LoginParam login, BindingResult bindingResult) throws NoSuchAlgorithmException {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser sysUser = null;
        List<SysUser> sysUserList = null;
        if (login.getType() == LoginTypeEnum.ACCOUNT) {//账号登录
            // 非空校验
            Optional.ofNullable(login.getLoginName()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("用户名不能为空"));
            Optional.ofNullable(login.getPassword()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("密码不能为空"));
            sysUserList = sysUserService.list(new QueryWrapper<SysUser>().lambda().eq(SysUser::getLoginName, login.getLoginName()));
            if (CollectionUtils.isEmpty(sysUserList)) {
                throw ExceptionResultEnum.ERROR.exception("用户不存在");
            }
            if (sysUserList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception("查出多个用户");
            }
            sysUser = sysUserList.get(0);

            if (!Objects.equals(sysUser.getPassword(), login.getPassword())) {
                throw ExceptionResultEnum.ERROR.exception("用户名或密码错误");
            }
        } else {//验证码模式
            Optional.ofNullable(login.getMobileNumber()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("手机号不能为空"));
            Optional.ofNullable(login.getCode()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("验证码不能为空"));

            SysConfig sysConfig = commonCacheService.addSysConfigCache(SystemConstant.SMS_NORMAL_CODE);
            Optional.ofNullable(sysConfig).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未配置万能短信验证码"));

            if (!Objects.equals(sysConfig.getConfigValue().trim(), login.getCode().trim())) {

            }
            sysUserList = sysUserService.list(new QueryWrapper<SysUser>().lambda().eq(SysUser::getMobileNumber, login.getMobileNumber()));
            if (CollectionUtils.isEmpty(sysUserList)) {
                throw ExceptionResultEnum.ERROR.exception("用户不存在");
            }
            if (sysUserList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception("查出多个用户");
            }
            sysUser = sysUserList.get(0);
        }
        Optional.ofNullable(sysUser).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未查询到用户"));
        return ResultUtil.ok(sysUserService.login(login.getPassword(), sysUser, AppSourceEnum.SYSTEM));
    }

    @ApiOperation(value = "登出")
    @RequestMapping(value = "/logout", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Result logout() throws NoSuchAlgorithmException {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(sysUserService.removeUserInfo(sysUser.getId(), false));
    }

    @ApiOperation(value = "文件上传接口")
    @RequestMapping(value = "/file/upload", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    public Result fileUpload(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
                             @ApiParam(value = "上传文件类型", required = true) @RequestParam UploadFileEnum type) throws Exception {
        BasicAttachment basicAttachment = null;
        try {
            basicAttachment = basicAttachmentService.saveAttachment(file, type);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            basicAttachmentService.deleteAttachment(basicAttachment);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return ResultUtil.ok(new EditResult(basicAttachment.getId(), basicAttachmentService.filePreview(basicAttachment.getPath())));
    }

    @ApiOperation(value = "下载导入模板")
    @RequestMapping(value = "/download_import_template", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public void downloadImportTemplate(@ApiParam(value = "模版类型", required = true) @RequestParam ImportTemplateEnum type) throws IOException {
        List<ImportTemplateEnum> templateEnums = Stream.of(ImportTemplateEnum.values()).map(m -> m).collect(Collectors.toList());
        if (!templateEnums.contains(type)) {
            throw ExceptionResultEnum.ERROR.exception("不支持的模板类型");
        }
        InputStream inputStream = this.getClass().getResourceAsStream(File.separator + SystemConstant.STATIC + File.separator + type.getTemplateName());
        // 导出
        importExportUtil.outputFile(inputStream, type.getFileName());
    }

    @ApiOperation(value = "文件下载接口")
    @RequestMapping(value = "/file/download", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    public Result fileDownload(@ApiParam(value = "任务id", required = true) @RequestParam Long taskId,
                               @ApiParam(value = "下载文件类型", required = true) @RequestParam DownloadFileEnum type) throws Exception {
        TBTask tbTask = tbTaskService.getById(taskId);
        Optional.ofNullable(tbTask).orElseThrow(() -> ExceptionResultEnum.TASK_NO_DATA.exception());
        String path = null;
        switch (type) {
            case IMPORT_FILE:
                path = tbTask.getImportFilePath();
                break;
            case REPORT_FILE:
                path = tbTask.getReportFilePath();
                break;
            case EXPORT_FILE:
                path = tbTask.getResultFilePath();
                break;
            default:
                break;
        }
        Optional.ofNullable(path).orElseThrow(() -> ExceptionResultEnum.PATH_NO_DATA.exception());
        return ResultUtil.ok(new EditResult(basicAttachmentService.filePreview(path)));
    }

    @ApiOperation(value = "文件预览接口")
    @RequestMapping(value = "/file/preview", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    public Result filePreview(@ApiParam(value = "附件id", required = true) @RequestParam Long id) throws Exception {
        return ResultUtil.ok(new EditResult(basicAttachmentService.filePreview(id)));
    }

    @ApiOperation(value = "发送验证码")
    @RequestMapping(value = "/get_verify_code", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    @Aac(auth = BOOL.FALSE)
    public Result getVerifyCode(@RequestBody LoginParam loginParam) {
        Optional.ofNullable(loginParam.getMobileNumber()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("手机号不能为空"));
        List<SysUser> userList = sysUserService.list(new QueryWrapper<SysUser>().lambda().eq(SysUser::getMobileNumber, loginParam.getMobileNumber()));

        if (!loginParam.getMobileNumber().matches(SystemConstant.REGULAR_EXPRESSION_OF_PHONE)) {
            throw ExceptionResultEnum.ERROR.exception("手机号[" + loginParam.getMobileNumber() + "]不符合输入规范");
        }

        //用户不存在
        if (CollectionUtils.isEmpty(userList)) {
            throw ExceptionResultEnum.ERROR.exception("手机号不存在");
        }
        if (userList.size() > 1) {
            throw ExceptionResultEnum.ERROR.exception("手机号绑定了多个用户");
        }

        SysUser sysUser = userList.get(0);
        if (!sysUser.getEnable()) {
            throw ExceptionResultEnum.ERROR.exception("手机号被禁用");
        }
        return ResultUtil.ok(basicVerifyCodeService.sendVeirfyCode(loginParam.getMobileNumber(), sysUser));
    }

    @ApiOperation(value = "查询用户权限")
    @RequestMapping(value = "/get_menu", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = MenuResult.class)})
    public Result getMenu() {
        return ResultUtil.ok(sysUserRoleService.listByUserId());
    }

    @ApiOperation(value = "获取服务器时间")
    @RequestMapping(value = "/get_system_time", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    @Aac(auth = BOOL.FALSE)
    public Result getSystemTime() {
        return ResultUtil.ok(System.currentTimeMillis());
    }

    @ApiOperation(value = "获取机构下所有用户")
    @RequestMapping(value = "/get_org_user_list", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = OrgDto.class)})
    public Result getOrgUserList() {
        return ResultUtil.ok(sysOrgService.listOrgTree(true));
    }
}
